"""Utilities for formatting time useful for when providing callables'."""

from __future__ import annotations

from ._utility import breakdown_seconds
from ._utility import format_seconds

__all__ = ("breakdown_seconds", "format_seconds")
